<?php

namespace App;

use Murdej\Duration;
use Nette\Application\UI;

/**
@property mixed $products
**/
class PackageEditForm extends DefaultForm {

	/** @var array */
	public $onClose;
	
	public function getProducts() {
		return Product::findBy(
			array(
				'isCategory' => false, 
				'packageId IS NULL',
				'code' => ''
			)
		)->order('position');
	}

	const durationPattern = '^ *([0-9]+[dmr] *)* *$';
	
	public function __construct(IContainer $parent = NULL, $name = NULL) {
		parent::__construct($parent, $name);
		// $this->jsHelper = 'person';
		$this->addHidden('id');
		$this->addText("name", "Název")->setRequired("Zadejte název pernametky");
		$this->addText("credit", "Kredit")->setType("number")
			->addCondition($this::FILLED)
			->addRule($this::FLOAT, 'Buď zadejte kredit číslem nebo nechte pole prázdné');
		$this->addText('validDuration', 'Doba platnosti', 10)
			->setRequired(FALSE)
			->addRule($this::PATTERN, 'Zadejte dobu nebo nechte pole prázdné', self::durationPattern);
        $this->addText('validDurationHint', 'Doba platnosti - upozornění', 10)
            ->setRequired(FALSE)
            ->addRule($this::PATTERN, 'Zadejte dobu nebo nechte pole prázdné', self::durationPattern);
		$this->addText('notifyBefore', 'Zvýraznit předem', 10)
			->setRequired(FALSE)
			->addRule($this::PATTERN, 'Zadejte dobu nebo nechte pole prázdné', self::durationPattern);
		// 	creditNegative
		$this->addCheckbox("creditChargeable", "Dobíjitelný kredit");
		$this->addCheckbox("creditNegative", "Kredit do mínusu");

		foreach($this->products as $product) {
			$subform = $this->addContainer('product'.$product->id);
			$subform->addHidden('name')->setValue($product->name);
			$subform->addText('amount')->setType('number')
				->addCondition($this::FILLED)
				->addRule($this::FLOAT, 'Buď zadejte množství číslem nebo nechte pole prázdné.');
			$subform->addCheckbox('fromCredit');
			$subform->addText('priceCredit')->setType('number')
				->addCondition($this::FILLED)
				->addRule($this::FLOAT, 'Buď zadejte cenu z kreditu nebo nechte pole prázdné.');
			$subform->addText('priceDirect')->setType('number')
				->addCondition($this::FILLED)
				->addRule($this::FLOAT, 'Buď zadejte cenu hotově číslem nebo nechte pole prázdné.');
				
		}
		
		$this->addSubmit('btnSave', 'Uložit')->onClick[] = [$this, 'formSave'];
		$this->addSubmit('btnCancel', 'Storno')
			->setValidationScope(null)
			->onClick[] = [$this, 'formCancel'];
		/*$this->addSubmit('btnDelete', 'Smazat')
			->setValidationScope(null)
			->onClick[] = $this->formDelete;*/
	}
	
	public function loadItem($id) {
		if (!(int)$id) unset($this['btnDelete']);
		$defaults = (int)$id
			? Package::formValues($id)
			: Package::defaults();
				
		$defaults['validDuration'] = $defaults['validDuration']
			? Duration::toString($defaults['validDuration'])
			: '';

        $defaults['validDurationHint'] = $defaults['validDurationHint']
            ? Duration::toString($defaults['validDurationHint'])
            : '';

        $defaults['notifyBefore'] = $defaults['notifyBefore']
			? Duration::toString($defaults['notifyBefore'])
			: '';
		
		// Glob::varbar($defaults);
		
		$this->setDefaults(
			$defaults
		);
	}
	
	public function formSave($button) {
		$values = $button->form->values;//->toArray();
		$values['validDuration'] = ($values['validDuration'] == '')
			? null
			: Duration::parse($values['validDuration']);

        $values['validDurationHint'] = ($values['validDurationHint'] == '')
            ? null
            : Duration::parse($values['validDurationHint']);

        $values['notifyBefore'] = ($values['notifyBefore'] == '')
			? null
			: Duration::parse($values['notifyBefore']);

		$package = array();
		$products = array();
		foreach($values as $k => $v) {
			if (substr($k, 0, 7) != 'product') {
				$package[$k] = $v;
			} else {
				$v['productId'] = (int)substr($k, 7);
				$products[$k] = $v;
			}
		}
		// Glob::varbar(array($values, $package, $products)); exit();
		$ePackage = (int)$values->id
			? Package::get((int)$values->id)
			: new Package();
		$package["credit"] = /*(float)$package["credit"]; */$package["credit"] === ""
			? null
			: (float)$package["credit"];

		$ePackage->fromArray($package);
		$ePackage->save();
		$package['id'] = $ePackage->id;
		/* if ((int)$values->id) { 
			Glob::context()->packages->update($package, $values['id']);
		} else {
			$dbp = Glob::context()->packages->insert($package);
			$package['id'] = $dbp->id;
		} */
		// Produkty
		foreach ($products as $product) {
			//if ($product['amount'] || $product['fromCredit'] || $product['priceCredit'] || $product['priceDirect']) {
				unset($product['name']);
				$product['amount'] = $product['amount'] === '' ? null : $product['amount'];
				$product['priceCredit'] = $product['priceCredit'] === '' ? null : $product['priceCredit'];
				$product['priceDirect'] = $product['priceDirect'] === '' ? null : $product['priceDirect'];
				$dbProduct = PackageProduct::getBy(array(
					'productId' => $product['productId'],
					'packageId' => $package['id']
				));
				if (!$dbProduct) $dbProduct = new PackageProduct();
				$product['packageId'] = $package['id'];
				$dbProduct->fromArray($product);
				$dbProduct->save();
			// }
		};
		$this->onClose();
	}

	public function formCancel($button) {
		$this->onClose();
	}
	
	public function formDelete($button) {
		
	}
}