<?php

namespace App;

// use Package

class ProductEditForm extends DefaultForm {

	/** @var array */
	public $onClose;
	
	public function __construct(IContainer $parent = NULL, $name = NULL) {
		parent::__construct($parent, $name);
		// $this->jsHelper = 'person';
		$this->addHidden('id');
		$this->addHidden('type');
		$this->addText("name", "Název")
			->setRequired("Zadejte název produktu");
		$this->addTextarea("description", "Popis");
		$this->addSelect("packageId", "Pernamentka", 
			Package::findBy([]) //Glob::context()->packages->findAll()
			->order('name')
			->selection
			->fetchPairs('id', 'name'))
			->setPrompt('Žádná pernamentka');
		$this->addText("price", "Cena")->setType("number")
			->setRequired("Zadejte cenu produktu")
			->addRule($this::FLOAT, 'Cena musí být číslo');
		$this->addSelect("taxRateId", "Výše DPH", TaxRate::findAll()->fetchPairs("id", "name"))
			->setRequired("Výše DPH musí být vybrána")
			->setPrompt('');

		$this->addText("position", "Pořadí")->setType("number")
			->setRequired("Zadejte pořadí")
			->addRule($this::FLOAT, 'Pořadí musí být číslo');
		$this->addCheckBox("enabled", "Povolen prodej");
		$this->addCheckBox("askPrice", "Volitelná cena");
		$this->addCheckBox("askCount", "Volitelné množství");
		$this->addCheckBox("askCustomName", "Volitelný název");
		$this->addCheckBox("stockable", "Skladová položka");
		$this->addSelect("productId", "Nadřazená kategorie");
		$this->addCheckBox("colorIsSet", "Nastavit barvu");
		$this->addText("color", "Barva")->setAttribute('type', 'color');
			//->setPrompt("Vyberte kategorii")
			//->setRequired("Vyberte kategorii");
		
		$this->addSubmit('btnSave', 'Uložit')->onClick[] = [$this, 'formSave'];
		// $this->addSubmit('btnPackage', 'Uložit a nastavit pernametku')->onClick[] = $this->formSave;
		$this->addSubmit('btnCancel', 'Storno')
			->setValidationScope(null)
			->onClick[] = [$this, 'formCancel'];
		/*$this->addSubmit('btnDelete', 'Smazat')
			->setValidationScope(null)
			->onClick[] = $this->formDelete;*/
	}
	
	public function loadItem($id, $parentId, $type, $packageId) {
		$this['productId']->setItems(Product::getCategoriesForSelect($id));
		if (!(int)$id) unset($this['btnDelete']);
		$defaults = (int)$id
			? Product::get($id)->toArray()
			: Product::defaults($parentId) 
				+ array(
					"productId" => $parentId, 
					'type' => (int)$type, 
					'packageId' => $packageId,
					'name' => $packageId ? Package::get($packageId)->name : ""
				);
		if ($defaults['type'] != Product::TypeProduct) {
			foreach(['price', 'packageId', 'askPrice', 'askCount', 'askCustomName', 'stockable'] as $n )
				unset($this[$n]);
		}
		$this->setDefaults(
			$defaults
		);
	}
	
	public function formSave($button) {
		$values = $button->form->values;//->toArray();
		// Glob::varbar($values);
		if ($values['productId'] == "root") $values['productId'] = null;
		if ($values['type'] == Product::TypeProduct)
			$values['packageId'] = $values['packageId'] ? (int)$values['packageId'] : null;
		
		/* if ((int)$values->id) { 
			Glob::context()->products->update($values, $values['id']);
		} else {
			Glob::context()->products->insert($values);
		}
		
		Glob::varbar($button->name); */
		$product = (int)$values->id 
			? Product::get((int)$values->id) 
			: new Product();

		unset($values['id']);
		$product->fromArray($values);
		$product->color = $values->colorIsSet ? $values->color : null;
		$product->save();

		$this->onClose();
	}

	public function formCancel($button) {
		$this->onClose();
	}
	
	public function formDelete($button) {
		
	}
}