<?php

namespace App;

/**
 * @dbTable
 * @property int $id (autoIncrement)
 * @property string $nameFirst [210]
 * @property string $nameLast [200]
 * @property string $note
 * @property string $code
 * @property string $contactPhone [45]
 * @property string $contactAddress [100]
 * @property ?\DateTime $dtBirth
 * @property ?string $rc [45]
 * @property bool $deleted [,,false]
 * @property ?MemberLevel $memberLevel (fk)
 * @related Prepaid $prepaids customerId
 * @defaultOrder nameLast, nameFirst
 **/
class Customer extends BaseModel 
{
	static $defaultOrder = array('nameFirst', 'nameLast');

	public function getFullName() 
	{
		return $this->nameFirst.' '.$this->nameLast;
	}
	
	public function getInstorePrepaids() 
	{
		return $this->prepaids->where('utValidFrom IS NULL');
	}

    public function getSuspendPrepaids()
    {
        return $this->prepaids->where('suspendValidLeft IS NOT NULL');
    }

	public function getActivePrepaids() 
	{
		return $this->prepaids->where('utValidFrom IS NOT NULL AND (utValidTo IS NULL OR utValidTo >= ?)', time());
	}

	// Repository
	public static function fulltext($str) 
	{
		$str = trim($str);
		if (preg_match('/'.Glob::$codeRE.'/', $str)) 
		{
			return self::findOneBy(array('code' => $str));
		}
		if ($str == "") return array();
		$res = self::findAll();
		$chunks = preg_split("/[\s]+/", $str);
		foreach($chunks as $chunk) 
		{
            $res->where("nameFirst LIKE ? OR nameLast LIKE ? OR nameFirst LIKE ? OR nameLast LIKE ?", 
                $chunk."%", 
                $chunk."%", 
                "% ".$chunk."%", 
                "% ".$chunk."%");
        }
		if ($res->count() > 0) 
		{
			return $res;
		}

		$res = self::findAll();
        foreach($chunks as $chunk) {
            $res->where("nameFirst LIKE ? OR nameLast LIKE ?", "%".$chunk."%", "%".$chunk."%");
		}

		return $res;
	}
	
	public static function isCodeUnique($code, $id) 
	{
		$code = trim($code);
		if (!$code) return true;
		$res = self::findAll();
		$res->where("code = ? AND id != ?", $code, $id);
		
		return !$res->exists();
	}

	public function getAge()
    {
        return $this->dtBirth ? self::computeAge($this->dtBirth) : null;
    }

	public static function computeAge($dt)
    {
        list($cy, $cmd) = self::dt2z_md(new \DateTime());
        list($ry, $rmd) = self::dt2z_md($dt);
        return $cy - $ry - ($rmd > $cmd ? 1 : 0);
    }

    public static function dt2z_md(\DateTime $dt)
    {
        return explode("-", $dt->format("Y-md"));
    }
}
