<?php

namespace App;

/** 
 * @dbTable
 * @property int $id (autoIncrement)
 * @property int $utCreated
 * @property int $utClosed (?)
 * @property Customer $customer (fk,?)
 * @property string $note (?)
 * @property User $user (fk)
 * @related ProductSell $productSells
 **/
class Directpaid extends BaseModel 
{
	public function getPriceDirectpaid() 
	{
		return $this->validSells->sum('priceDirectpaid');
	}

    public function getPriceCachless()
    {
        return $this->validSells->sum('priceCachless');
    }

    public static function defaults()
	{
		return array(
			'utCreated' => time(),
			'userId' => Glob::currentAccountId(),
			'utClosed' => null,
			'customerId' => null,
			'note' => '',
		);
	}

	public function getIsOpen() 
	{
		return $this->utClosed == null;
	}

	public function isOneshot() 
	{
		return !$this->customerId && !$this->note;
	}

	public function getIdentify() 
	{
		if ($this->customerId) {
			return 
				$this->customer->fullName
				.($this->customer->note ? ' ('.$this->customer->note.')' : '')
                .($this->customer->dtBirth || $this->customer->memberLevelId ? " / " : "")
                .($this->customer->memberLevel ? $this->customer->memberLevel->name." " : "")
                .($this->customer->dtBirth ? $this->customer->getAge()." " : "");
		}
		if ($this->note) return $this->note;
		return 'Jednorázový prodej';
	}

	public function add(ProductPrice $pp, $note = null) 
	{
		$pso = new ProductSell(); // $this->context->productSells->insert($ps);
		$pso->productId = $pp->productId;
		$pso->prepaidId = $pp->prepaidId;
		$pso->packageProductId = $pp->packageProductId;
		$pso->directpaidId = $this->id;
		$pso->utSell = time();
		$pso->createUserId = Glob::currentAccountId();
		$pso->customName = $pp->customName;
		$pso->bonusCredit = $pp->bonusCredit;
		$pso->priceCachless = $pp->priceCachless ? floatval($pp->priceCachless) : null;
		$msg = '';
		switch($pp->type) 
		{
			case ProductPrice::TRegular:
			case ProductPrice::TPDiscount:
				$pso->priceDirectpaid = $pp->price;
				$msg = 'Přidáno na lístek: '.$pp->product->name.' v ceně '.Glob::formatPrice($pp->price).'.';
				break;
			case ProductPrice::TPCreditNegative:
			case ProductPrice::TPCredit:
				// $ps['priceCredit'] = $pp->price;
				$pr = Prepaid::get($pp->prepaidId);
				$ucr = $pr->useCredit($pp->price);
				$pso->priceCreditCA = $ucr->creditCA;
				$pso->priceCreditCL = $ucr->creditCL;
				$pso->priceCreditExistsCA = $ucr->creditExistsCA;
				$pso->priceCreditExistsCL = $ucr->creditExistsCL;
				$msg = 'Zaplaceno z kreditu pernametky '.$pr->package->name.': '.$pp->product->name.' v ceně '.Glob::formatPrice($pp->price).'.'; 
				// .($isClose ? 'Pernamentka už neobsahuje kredit a proto byla uzařena.' : "");
				break;
			case ProductPrice::TPackage:
				$pso->productAmount = 1;
				$pr = Prepaid::get($pp->prepaidId);
				$isClose = $pr->useProduct($pp->packageProductId);
				$msg = 'Vybráno z pernamentky '.$pr->package->name.': '.$pp->product->name.'.'
					.($isClose ? 'Pernamentka už neobsahuje další produkty a proto byla uzařena.' : "");
			break;
		}
		
		$pso->save();

		$product = Product::get($pso->productId);
		if ($product->stockable)
		{
			$sch = new StockChange();
			$sch->productId = $product->id;
			$sch->utChange = $pso->utSell;
			$sch->productSellId = $pso->id;
			$sch->changeAmount = -1;
			$sch->save();
		}

		if ($note) $pso->addNote($note);

		return $msg;
	}

	public function getValidSells() 
	{
		return $this->productSells
			->where('cancelUt IS NULL');
	}

	public function close() 
	{
		// Zkontroluj jestli uz neni uzavren
		if (!$this->utClosed) {
			$this->utClosed = time();
			$this->save();
			$messages = array();
			$prepaidIds = [];
			// Připrav balicky
			$validSells = $this->validSells;
			foreach($validSells as $ps) {
                /** @var ProductSell $ps */
				$usedIds[] = $ps->id;
				$product = $ps->product; // $ps->ref('product', 'productId');
				if ($product->packageId) 
				{
					$prepaid = $product->package->makePrepaid($this->customerId);
					$messages[] = "Byla přidána pernametka ".$prepaid->identify;
				}
				if ($ps->prepaidId) $prepaidIds[] = $ps->prepaidId;
				//Dobiti kreditu
				if ($product->code == "creditChange") 
				{
					$pp = $ps->prepaid;
					$bonusStr = $ps->bonusCredit ? ' + '.$ps->bonusCredit.' (bonus)' : '';
                    $cachlessStr = $ps->priceCachless ? ' + '.$ps->priceCachless.' (bezhotovostně)' : '';
					$pp->addCredit($ps->priceDirectpaid, true);
                    if ($ps->bonusCredit) $pp->addCredit($ps->bonusCredit, false);
                    if ($ps->priceCachless) $pp->addCredit($ps->priceCachless, true);
					$messages[] = "U pernametky ".$pp->identify.' byl navýšen kredit o '.($ps->priceDirectpaid + $ps->priceCachless + $ps->bonusCredit)
                        .$bonusStr.'. aktuální kredit je '.$pp->creditLeft.'.';
					$pp->addChange('Navýšen kredit o '.$ps->priceDirectpaid.'  '.$bonusStr.' '.$cachlessStr.' aktuální kredit je '.$pp->creditLeft.'. #'.$ps->id);
				}
			}
			// Ukonci neplatne
			foreach(array_unique($prepaidIds) as $prepaidId) 
			{
				$prepaid = Prepaid::get($prepaidId);
				if ($prepaid->closeIfEmpty()) {
					$messages[] = "Pernametka ".$prepaid->identify.' byla automaticky uzavřena protože už neobsahue kredit ani produkty.';
				}
			}
			// EET
			$rec = new Receipt();
			$rec->cachRegisterId = 1;
			$rec->prepare($validSells);
			if ($rec->state !== Receipt::State_empty) {
				// bude se vytvářet účtenka
				$messages[] = (object)[
					"type" => "eet",
					"receiptId" => $rec->id
				];
			}
		}
		else 
		{
			$messages[] = "Tento lístek byl uzavřen už dříve.";
		}
		return $messages;
	}

	// Repo
	public static function getOpenList() 
	{
		return self::findBy('utClosed IS NULL');
	}

	// Vytvori jednorazovy listek
	public static function oneshot() 
	{
		$dp = new Directpaid();
		$dp->fromArray(Directpaid::defaults());
		// dump(Directpaid::defaults());
		// dump($dp);
		$dp->save();
		
		return $dp;
		// return $this->insert(Directpaid::defaults());
	}
	
	// Najde aktualni listek zakaznika, pokud nema vytvori novy
	public static function byCustomer($customerId) 
	{
		$dp = Directpaid::getBy([
			'utClosed IS NULL',
			'customerId' => $customerId
		]);
		if ($dp == null)
		{
			$dp = new Directpaid();
			$dp->fromArray(Directpaid::defaults());
			$dp->customerId = $customerId;
			$dp->save();
		}

		return $dp;
		/* $ls = $this->getTable()->where(array(
			'utClosed IS NULL',
			'customerId' => $customerId
		));
		if ($ls->count() > 0) {
			return $ls->fetch();
		} else {
			$values = Directpaid::defaults();
			$values['customerId'] = $customerId;
			return $this->insert($values);
		} */
	}
	
	public static function addNew($note) 
	{
		$dp = new Directpaid();
		$dp->fromArray(Directpaid::defaults());
		$dp->note = $note;
		$dp->save();

		return $dp;
		/* $values = Directpaid::defaults();
		$values['note'] = $note;
		return $this->insert($values); */
	}

}