<?php

namespace App\Presenters;

use Nette\Utils\Arrays;
use Nette\Diagnostics\Debugger;
use App;
use App\Directpaid;
use App\Product;
use App\ProductSell;
use App\Customer;
use App\Prepaid;
use App\Receipt;
use Murdej\ProcI;

/**
@property App\Directpaid $directpaid
**/
class CashboxPresenter extends BasePresenter
{
	// --- Start ---
	public function renderDefault() {
		$this->template->directpaidList = Directpaid::getOpenList();
	}

	public function actionDefault() {
		$this->prepaidId = null;
	}
	
	public function createComponentCustomerSearchForm() {
		$form = new App\DefaultForm();
		$form->addText("q")->setRequired('Zadejte část jména nebo kód');
		$form->addSubmit("btnSearch", "Hledat");
		$form->onSuccess[] = [ $this, 'customerSearchFormSubmited' ];
		
		return $form;
	}
	
	public function customerSearchFormSubmited($form) {
		$res = Customer::fulltext($form->values->q);
		if (get_class($res) == 'App\Customer') {
			$this->redirect('sell', 'c'.$res->id);
		} else {
			$this->template->data = $res;
		}
		//App\Glob::varbar($res);
	}
	
	public function createComponentNewDirectpaid() {
		$form = new App\DefaultForm();
		$form->addSubmit("btnNew", "Prodej na lístek");
		$form->addText("note")->setRequired("Zadejte poznámku");
		$form->onSuccess[] = [$this, 'newDirectpaidFormSubmited'];
		$form->setLayout('inline');
		
		return $form;
	}

	public function createComponentUpdateDirectpaid() {
		$form = new App\DefaultForm();
		$form->addText("note")->setDefaultValue($this->directpaid->note); //->setRequired("Zadejte poznámku");
		$form->addSubmit("btnUpdate", "Změnit poznámku");
		$form->onSuccess[] = [$this, 'updateDirectpaidFormSubmited'];
		$form->setLayout('inline');
		
		return $form;
	}

	public function newDirectpaidFormSubmited($form) {
		$directpaid = Directpaid::addNew($form->values->note);
		$this->redirect('sell', 'd'.$directpaid->id);
	}
	
	public function updateDirectpaidFormSubmited($form) {
		$this->directpaid->note = $form->values->note;
		$this->directpaid->save();
		/* $dp = Directpaid::get()
		App\Glob::context()->directpaids->update(array(
			'note' => $form->values->note
		), $this->directpaid->id); */
		$this->redirect('sell', 'd'.$this->directpaid->id);
	}
	//$prepaid
	// --- Produkty ---
	protected $directpaidId = null;
	
	public function actionSell($id, $rootId = null) {
		if ($id[0] == 'd') {
			$this->directpaidId = (int)substr($id, 1);
			if ($this->directpaidId) {
				// Listek
				$this->directpaid = Directpaid::get($this->directpaidId);
			} else {
				// Jednorazovy nakup
				$directpaid = Directpaid::oneshot();
				$this->redirect('this', 'd'.$directpaid->id);
			}
		} else if ($id[0] == 'c') {
			// Zakaznik
			$this->directpaid = Directpaid::byCustomer( (int)substr($id, 1));
			$this->directpaidId = $this->directpaid->id;
		}
		// Kontrola jestli neni listek uzavreny
		if (!$this->directpaid->isOpen) {
			$this->flashMessage('Vybraný lístek je uzavřený, další nákup není možný.');
			$this->redirect('Cashbox:');
		}
		$this->template->customerName = $this->directpaid->identify;
		$this->template->prepaidId = null;
		$this->template->packageId = null;
		$this->template->rootProduct = $rootId ? Product::get($rootId) : null; 
	}

	function getDirectpaid() {
		return $this->template->directpaid;
	}

	function setDirectpaid($value) {
		$this->template->directpaid = $value;
	}
	
	protected $productsByParent = null;
	
	public function categoryContent($category) {
		if (!$this->productsByParent) {
			$this->productsByParent = array();
			$l = Product::findBy(['enabled' => true])->order('position');
			if (!$this->directpaid->customerId) $l = $l->where('packageId IS NULL');
			foreach($l as $prod) {
				$productsByParent[(int)$prod->productId][] = $prod;
			}
			// Debugger::barDump($this->productsByParent);
		}
		$cid = $category ? $category->id : 0;
		return isset($productsByParent[$cid])
			? $productsByParent[$cid]
			: array();
		/* $content = $category
			? $category->related('product')
			: App\Glob::context()->products->findAll()->where('productId IS NULL');
		// Nezakaznim s nemuze koupit pernamentku
		if (!$this->directpaid->customerId) $content->where('packageId IS NULL');
		return $content->order('position')->where(array('enabled' => true)); */
	}
	
	/** @persistent */
	public $prepaidId = null;
	
	/** @persistent */
	public $packageId = null;

	public function renderSell() {
		$this->template->prepaidId = $this->prepaidId; 
		$this->template->packageId = $this->packageId;
		$this->template->fixScroll = "sell";

	}
	
	public function handleShowPrepaid($prepaidId = null, $packageId = null) {
		$this->prepaidId = $prepaidId; 
		$this->packageId = $packageId;
		App\Glob::varbar(array($prepaidId, $packageId));
	}
	
	public function handleClosePrepaid() {
		$this->prepaidId = null;
		$this->packageId = null;
	}
	
	public function handleActivatePrepaid($prepaidId) {
		$pp = Prepaid::get($prepaidId);
		$pp->utValidFrom = time();
		if ($pp->package->validDuration) {
			$pp->utValidTo = time() + $pp->package->validDuration;
		}
		$pp->save();
		
		$this->flashMessage('Pernamentka '.$pp->package->name.' byla aktivována a je platná do '.App\Glob::formatDate($pp->utValidTo));
		$this->redirect('this');
	}
	
	public function handleDeactivatePrepaid($prepaidId) {
		$pp = Prepaid::get($prepaidId);
		$pp->utValidTo = time();
		$pp->save();
		
		$this->flashMessage('Pernamentka '.$pp->package->name.' byla deaktivována');
		$this->redirect('this');
	}
	
	public function handleAddProduct($code) {
		$pp = new App\ProductPrice($code);
		$product = $pp->product;
		App\Glob::varbar($product);
		if ($product->askCount || $product->askPrice) {
			$this->template->sellCode = $code;
			$form = $this->getComponent('productSellForm');
			$form->setDefaults(array(
				'count' => 1,
				'price' => $pp->price,
				'code' => $code,
			));
			if (!$product->askPrice || $pp->type == App\ProductPrice::TPackage) $form['price']->setDisabled();
			if (!$product->askCount) $form['count']->setDisabled();
			if (!$product->askCustomName) {
				$form['customName']->setDisabled();
			} else {
				$form['customName']->setRequired("Zadej název zboží");
			}
		} else {
			$this->flashMessage($this->directpaid->add($pp));
			$this->redirect('this');
		}
	}
	
	public function createComponentProductSellForm() {
		$form = new App\DefaultForm();
		$form->AddText('customName', 'Zboží');
		$form->AddText('count', 'Počet')->setType('number');
		$form->AddText('price', 'Cena')->setType('number');
		$form->AddTextArea('note', 'Poznámka');
		$form->AddHidden('code');
		$form->AddSubmit('btnSubmit', 'Prodat');
		$form->onSuccess[] = [$this, 'productSellFormSubmited'];
		
		return $form;
	}
	
	public function productSellFormSubmited($form) {
		$pp = new App\ProductPrice($form->values->code);
		$product = $pp->product;
		if ($product->askPrice) $pp->price = $form->values->price;
		if ($product->askCustomName) $pp->customName = $form->values->customName;
		$c = $product->askCount ? $form->values->count : 1;
		for ($i = 0; $i < $c; $i++)
			$this->flashMessage($this->directpaid->add($pp, $form->values->note));
		$this->redirect('this');
	}
	
	public function createComponentPackage() {
		return new App\ComPackage();
	}
	
	// --- Uzavreni / detail listku ---
	public function actionDirectpaidClose($id) {
		$this->directpaid = Directpaid::get((int)$id);
		$this->template->showClose = false;
		$this->template->cancelProductSell = null;
		
	}
	/*public function renderDirectpaidClose($id) {
		
	}*/
	public function handleClose($confirmed = false) {
		if ($confirmed || $this->directpaid->priceDirectpaid <= 0) {
			$messages = $this->directpaid->close();
			$this->flashMessages(ProcI::from($messages)->filter(function($v) { return is_string($v); })->toArray());
			$this->flashMessage(
				'Lístek: '.$this->directpaid->identify.' byl uzavřen'
				.($this->directpaid->priceDirectpaid <= 0 
					? ' automaticky protože částka k platbě byla nulová.' 
					: '.'
				)
			);
			$this->redirect('this');
		} else {
			$this->template->showClose = true;
		}
	}

	public function handleCancel($productSellId) {
		$this->template->cancelProductSell = ProductSell::get($productSellId);
		$this->getComponent('cancelForm')->setDefaults([
			'productSellId' => $this->template->cancelProductSell->id
		]);
	}

	public function createComponentCancelForm() {
		$form = new App\DefaultForm();
		$form->addHidden('productSellId');
		$form->addTextArea('cancelNote', 'Důvod storna', 60, 3)->setRequired('Zadejte důvod.');
		$form->addSubmit('btnCancel', 'Stornovat položku');
		$form->onSuccess[] = [$this, 'cancelFormSubmited'];
		return $form;
	}

	public function cancelFormSubmited($form) {
		$ps = ProductSell::get($form->values->productSellId);
		$ps->cancel($form->values->cancelNote);
		$this->flashMessage('Položka byla stornována');
		$this->redirect('this');
	}

	public function createComponentProductForm() {
		$form = new App\DefaultForm();
		$form->addHidden('productSellId');
		$form->addTextArea('cancelNote', 'Důvod storna', 60, 3)->setRequired('Zadejte důvod.');
		$form->addSubmit('btnCancel', 'Stornovat položku');
		$form->onSuccess[] = $this->cancelFormSubmited;

		return $form;
	}
	
	public function createComponentOtherForm() {
		$form = new App\DefaultForm();
		$form->addHidden('productSellId');
		$form->addTextArea('cancelNote', 'Důvod storna', 60, 3)->setRequired('Zadejte důvod.');
		$form->addSubmit('btnCancel', 'Stornovat položku');
		$form->onSuccess[] = $this->cancelFormSubmited;

		return $form;
	}
	// Detail zákazníka
	
	public function actionCustomerDetail($id) {
		$this->template->customer = Customer::get($id);
		$this->template->data = 
			ProductSell::findBy(['directpaid.customerId' => $id])
			->order('utSell');
		switch($this->backTo)
		{
			case 'admin-customer':
				$this->template->backLink = $this->link('Admin:customer');
				break;
			default:
				$this->template->backLink = $this->link('Cashbox:sell', 'c'.$id);
				break;
		}
	}

	public function actionReceiptPaper(int $id)
	{
		# code...
	}

	/** @persistent */
	public $backTo = null;

	public function renderCustomerDetail() {
		$this->template->prepaidId = $this->prepaidId; 
		$this->template->packageId = $this->packageId;
		$this->template->fixScroll = "customerDetail";
	}

	public function actionTest(?int $id = null)
	{
		if (!$id) $id = Directpaid::findAll()->order("id DESC")->limit(1)->fetch()->id;
		$rec = new Receipt();
		$rec->cachRegisterId = 1;
		$rec->prepare(Directpaid::get($id)->getValidSells());
		$rec->send();
		$rec->save();
	}
}