<?php

namespace App\Presenters;

use App\StockChange;
use App\StockFill;
use App\Glob;
use App\Product;
use Nette\Application\UI\Form;
use Nette\Utils\Strings;
use Nette;
/**
 * Sklad.
 * @property mixed $productStat
 */
class StockPresenter extends BasePresenter
{
	// Statistika - úvodní stránka
		protected $utTo = null;

		protected $utFrom = null;

		public function actionDefault($utFrom = null, $utTo = null) 
		{
			$this->utFrom = $utFrom;
			$this->utTo = $utTo;
		}

		public function renderDefault() 
		{
			$this->template->data = $this->productStat;
		}

		public function getProductStat()
		{
			if ($this->_productStat === null) 
			{
				$res = [];
				foreach(StockChange::productStat($this->utFrom, $this->utTo) as $row) $res[] = $row;
				$this->_productStat = $res;
			}
			return $this->_productStat;
		}
		protected $_productStat = null;

		public function createComponentStatForm()
		{
			$form = new \App\DefaultForm();
			$form->addDateInput('utFrom', 'Datum od');
			$form->addDateInput('utTo', 'do');
			$form->addSubmit('btnSubmit', 'Zobrazit');
			$form->onSuccess[] = [$this, 'statFormSubmited'];
			$form->setLayout('inline');
			
			return $form;		
		}

		public function statFormSubmited($form)
		{
			$values = $form->getValues();
			$this->utFrom = $values->utFrom;
			$this->utTo = $values->utTo;
		}

	// Historie produktu
		public function actionProductChanges($id, $p = 1)
		{
			$changes = StockChange::productHistory($id);

			$paginator = new Nette\Utils\Paginator;
			$paginator->setItemCount($changes->count()); 
			$paginator->setItemsPerPage(50);
			$paginator->setPage($p); 

			$this->template->changes = $changes->limit($paginator->getLength(), $paginator->getOffset());
			$this->template->paginator = $paginator; 
			$this->template->product = Product::get($id);
		}
	
	// Naskladnení
		public function actionStockFill()
		{
			$this->template->data = $this->productStat;
		}

		public function createComponentStockFillForm()
		{
			$form = new \App\DefaultForm();
			foreach($this->productStat as $row)
			{
				$form->addText("change_".$row->productId)
					->setType('number')
					->addCondition(Form::FILLED)
					->addRule(Form::FLOAT, 'Buď zadejte množství nebo nechte pole prázdné.');
			}
			$form->addTextarea("note", "Poznámka");
			$form->addSubmit("btFill", "Naskladnit");
			$form->onSuccess[] = [$this, "stockFillFormSubmited"];

			return $form;
		}

		public function stockFillFormSubmited($form)
		{
			$changes = [];
			foreach ($form->getValues() as $name => $value) if (Strings::startsWith($name, "change_"))
			{
				$value = (int)trim($value);
				if ($value)
					$changes[substr($name, 7)] = $value;
			}
			if ($changes)
			{
				$sf = new StockFill();
				$sf->utFill = time();
				$sf->note = $form->getValues()->note;
				$sf->userId = Glob::currentAccountId();
				$sf->save();

				foreach($changes as $productId => $change)
				{
					$sch = new StockChange();
					$sch->changeAmount = $change;
					$sch->productId = $productId;
					$sch->stockFillId = $sf->id;
					$sch->utChange = $sf->utFill;
					$sch->save();
				}

				$this->flashMessage("Naskladnění bylo provedeno");
				$this->redirect("stockFillDetail", $sf->id);
			} else $this->flashMessage("Nebyla zadáno množstní ani u jedné položky, není co (vy)naskladnit");
			//todo: flash, redir, ...
		}

	// Detail naskladnění
		public function actionStockFillDetail($id)
		{
			$stockFill = StockFill::get($id);
			$this->template->stockFill = $stockFill;
			$this->template->data = $stockFill->changes;

			$this->menu->selectByUrl($this->link('Stock:stockFillHistory')); //'Stock:stockFillHistory'; 
		}

	// Historie naskladnění
		public function actionStockFillHistory($p = 1)
		{
			$items = StockFill::findAll()->order('utFill DESC');

			$paginator = new Nette\Utils\Paginator;
			$paginator->setItemCount($items->count()); 
			$paginator->setItemsPerPage(50);
			$paginator->setPage($p); 

			$this->template->items = $items->limit($paginator->getLength(), $paginator->getOffset());
			$this->template->paginator = $paginator;
		}
}