<?php

namespace App\Service;

use App\Receipt as AppReceipt;
use App\ReceiptTaxItem;
use Nette\Utils\DateTime;
use SlevomatEET\Client;
use SlevomatEET\Configuration;
use SlevomatEET\Cryptography\CryptographyService;
use SlevomatEET\Driver\GuzzleSoapClientDriver;
use SlevomatEET\EvidenceEnvironment;
use SlevomatEET\Receipt;

class EETSender 
{
	public $isProd = false;

	public $certBasePath = APP_DIR."/../data/certs";

	public function send(\App\Receipt $receipt, bool $verify = false)
	{
		/** @var \App\CachRegister $cachRegister */
		$cachRegister = $receipt->cachRegister;
		$company = $cachRegister->company;
		$crypto = new CryptographyService(
			$this->certBasePath."/".$company->certFilePrivate, //'cesta k privátnímu klíči', 
			$this->certBasePath."/".$company->certFilePublic, //'cesta k veřejnému klíči', 
			$company->certPassword //'heslo privátního klíče (nebo prázdný string pokud bez hesla)'
		);
		$configuration = new Configuration(
			$company->vatUID, // 'DIČ poplatníka',
			$cachRegister->workshopUID, // 'Identifikace provozovny ',
			$cachRegister->cashRegisterUID, // 'Identifikace pokladního zařízení',
			EvidenceEnvironment::get($this->isProd
				? EvidenceEnvironment::PRODUCTION
				: EvidenceEnvironment::PLAYGROUND), // nebo EvidenceEnvironment::get(EvidenceEnvironment::PRODUCTION) pro komunikaci s produkčním systémem
			$verify // zda zasílat účtenky v ověřovacím módu
		);
		$client = new Client($crypto, $configuration, new GuzzleSoapClientDriver(new \GuzzleHttp\Client()));

		$values = [
			"priceZeroVat" => null,
			"priceStandardVat" => null,
			"vatStandard" => null,
			"priceFirstReducedVat" => null,
			"vatFirstReduced" => null,
			"priceSecondReducedVat" => null,
			"vatSecondReduced" => null,
			"priceTravelService" => null,
			"priceUsedGoodsStandardVat" => null,
			"priceUsedGoodsFirstReducedVat" => null,
			"priceUsedGoodsSecondReducedVat" => null,
			"priceSubsequentSettlement" => null,
			"priceUsedSubsequentSettlement" => $receipt->totalCreditCA,
		];

		foreach($receipt->receiptTaxItems as $rti)
		{
			/** @var ReceiptTaxItem $rti */
			$k = $rti->taxRate->field;
			if ($k[0] == "{")
			{
				$opts = json_decode($k);
				$kPriceNT = $opts->priceNTName;
				$kVat = $opts->priceTaxName;
			} 
			else 
			{
				$kPriceNT = "price${k}Vat";
				$kVat = "vat${k}";
			}
			$values[$kPriceNT] = $rti->priceNT;
			$values[$kVat] = $rti->priceTax;
		}

		foreach($values as $k => $v) $values[$k] = (int)($v * 100);
		$eRecTotal = (int)(($receipt->totalCache + $receipt->totalCreditCA) * 100);
		dump($eRecTotal);
		dump($values);
		$eReceipt = new Receipt(
			$receipt->state == AppReceipt::State_new,
			null,
			$receipt->counterValue,
			\DateTimeImmutable::createFromMutable($receipt->dtCreate),
			// new \DateTimeImmutable(), // ---- new \DateTimeImmutable('2016-11-01 00:30:12'),
			$eRecTotal,
			$values["priceZeroVat"],
			$values["priceStandardVat"],
			$values["vatStandard"],
			$values["priceFirstReducedVat"],
			$values["vatFirstReduced"],
			$values["priceSecondReducedVat"],
			$values["vatSecondReduced"],
			$values["priceTravelService"],
			$values["priceUsedGoodsStandardVat"],
			$values["priceUsedGoodsFirstReducedVat"],
			$values["priceUsedGoodsSecondReducedVat"],
			$values["priceSubsequentSettlement"],
			$values["priceUsedSubsequentSettlement"]
	
		);

		$eReceipt->getPriceForSubsequentSettlement();

		$receipt->dtSend = new \DateTime();
		try {
			$response = $client->send($eReceipt);
			$receipt->bkp = $response->getBkp();
			$receipt->fik = $response->getFik();
			$receipt->pkp = base64_encode($response->getRequest()->getPkpCode());
			$receipt->state = AppReceipt::State_sent;
		} catch (\SlevomatEET\FailedRequestException $e) {
			$receipt->pkp = base64_encode($e->getRequest()->getPkpCode());
			$receipt->bkp = $e->getRequest()->getBkpCode();
			$receipt->errorMessage = $e->__toString();
		} catch (\SlevomatEET\InvalidResponseReceivedException $e) {
			$receipt->pkp = base64_encode($e->getResponse()->getRequest()->getPkpCode());
			$receipt->bkp = $e->getResponse()->getRequest()->getBkpCode();
			$receipt->errorMessage = $e->__toString();
		}
	}
}

