<?php

namespace Murdej\Forms;

use DateTime;
use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;

// require_once LIBS_DIR . '/Nette/Forms/Controls/TextInput.php';

/**
 * DatePicker input control.
 *
 * @author     Murdej, Tomáš Kraina, Roman Sklenář
 * @package    Nette\Extras
 */
class DateInputDT extends \Nette\Forms\Controls\TextInput		
{

	var $profile = null;
	
	var $options = array();
	
	/**
	 * @param  string  label
	 * @param  int  width of the control
	 * @param  int  maximum number of characters the user may enter
	 */
	public function __construct($label, $profile = 'near', $cols = 15, $maxLenght = 10, $minDate = null, $maxDate = null)
	{
		parent::__construct($label, $cols, $maxLenght);
		$this->profile = $profile;
	}


	/**
	 * Returns control's value.
	 * @return ?DateTime
	 */
	public function getValue()
	{
		return $this->value ? new DateTime($this->value) : null;
	}


	/**
	 * Sets control's value.
	 * @param  string|DateTime
	 * @return void
	 */
	public function setValue($value)
	{
		// bdump(["setValue", $value]);
		// if ($value instanceof \DateTime)  $value = $value->format("Y-m-d\\TH:i:s");
        if ($value instanceof \DateTime)  $value = $value->format("Y-m-d");
		parent::setValue($value);
	}


	/**
	 * Generates control's HTML element.
	 * @return Html
	 */
	public function getControl() : \Nette\Utils\Html
	{		
		$control = parent::getControl();
		$control->attrs['type'] = "date";
		$control->attrs['value'] = $this->value;
		// bdump(["getControl", $this]);
		return $control;
	}
	
	static function registerNette() {
		// FormContainer::extensionMethod('addDateInput', callback(__CLASS__, 'addDateInput')); // v PHP 5.3
		FormContainer::extensionMethod('addDateInputDT', array(__CLASS__, 'addDateInput')); // v PHP 5.3
	}

	static function register() 
	{
		FormContainer::extensionMethod('addDateInput', array(__CLASS__, 'addDateInput')); // v PHP 5.3
	}


	static function addDateInput($_this, $name, $label, $profile = 'near', $cols = 12, $maxLength = 10)
	{
		// \Murdej\PageHead::addOnLoad('$(".dateinput").datepicker({ showButtonPanel: true });', 1, true);
		// \Nette\Diagnostics\Debugger::barDump(\Murdej\PageHead::getInstance());
		return $_this[$name] = new DateInputDT($label, $profile, $cols, $maxLength);
	}
}
