<?php

namespace Murdej\Forms;

use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;
use Nette\Utils\Html;
use Murdej\ProcI;
use Nette\Utils\Arrays;
use Nette\Utils\DateTime;
use DateTimeInterface;

/**
* Datetime range control.
*
* @author     Murdej
* @package    Nette\Extras
*/
class DateTimeIntervalControl extends \Nette\Forms\Controls\BaseControl
{
	use \Nette\SmartObject;

	/**
	 * Režim zarovnání s, m, h, d
	 */
	public $mode = 'm';

	// protected $_value;

	public $filterNull = false;

	static $modeOpts = [
		's' => [ 'format' => 'd.m.Y H:i:s', 'jsDateFormat' => 'dd.mm.yy', 'jsTimeFormat' => 'hh:mm:ss'],
		'm' => [ 'format' => 'd.m.Y H:i', 'jsDateFormat' => 'dd.mm.yy', 'jsTimeFormat' => 'hh:mm'],
		'd' => [ 'format' => 'd.m.Y', 'jsDateFormat' => 'dd.mm.yy', 'jsTimeFormat' => null],
	];

	public function getValue() //: DateTimeInterval
	{
		$this->value->roundMode = &$this->mode;
		return $this->value;
	}

	public function setValue(/*DateTimeInterval */$value)
	{
		if ($value instanceof DateTimeInterval)
		{
			$this->value = $value;
            $this->value->roundMode = &$this->mode;
		}
		else if (is_array($value))
		{
			$this->value = new DateTimeInterval($value);
			$this->value->roundMode = &$this->mode;
			// list($this->value->dtFrom, $this->value->dtTo) = $value;
		}
	}

	public function __construct($label)
	{
		parent::__construct($label);
		$this->value = new DateTimeInterval();
	}

	protected function makeSubInput(string $tail, $value, array $attributes = [])
	{
		$subInput = clone parent::getControl();
		$subInput->addAttributes([
			"mu-id" => $tail,
			"value" => $value,
			'autocomplete' => "off",
			'name' => $this->getHtmlName().self::NAME_SEPARATOR.$tail,
			'id' => $this->getHtmlId().self::NAME_SEPARATOR.$tail,
		] + $attributes);

		return $subInput;
	}
	/**
	* Generates control's HTML element.
	* @return Html
	*/
	public function getControl() : Html
	{
		$el = Html::el('div')
			->addAttributes([
				"mu-widget" => "MurdejFormDateTimeInterval",
				'class' => 'MurdejFormDateTimeInterval'.($this->filterNull ? '-null' : '').' form-control',
				'mu-params' => json_encode([
					'dateFormat' => self::$modeOpts[$this->mode]['jsDateFormat'],
					'timeFormat' => self::$modeOpts[$this->mode]['jsTimeFormat'],
					'mode' => $this->mode,
				])
			]);
		$dtattrs = ['type' => /*'datetime-local'*/'text', 'step' => 1];
		if ($this->filterNull) 
		{
			$el->addHtml(
				Html::el('label')
					->addHtml($this->makeSubInput(
						'isNull', 
						'1',
						['type' => 'checkbox', 'checked' => $this->value->isNull]
					))
					->addText('Nevyplněno')
					->setClass('is-null')
			);
		}
		$el->addHtml(
			Html::el('label')
				->addHtml(Html::el('span')->setText('Od'))
				->addHtml($this->makeSubInput(
					'dtFrom', 
					$this->value->getFixDtFrom() ? $this->value->getFixDtFrom()->format(self::$modeOpts[$this->mode]['format']) : '',
					$dtattrs
				))
				->setAttribute("mu-id", "fromLabel")
			)
			->addHtml(
			Html::el('label')
				->addHtml(Html::el('span')->setText('do'))
				->addHtml($this->makeSubInput(
					'dtTo', 
					$this->value->getFixDtTo() ? $this->value->getFixDtTo()->format(self::$modeOpts[$this->mode]['format']) : '',
					$dtattrs
				))
				->setAttribute("mu-id", "toLabel")
			);

		return $el;
	}

	protected function parseDatetime(/*?string*/ $src) /* : ?DateTime */
	{
		if (!$src) return null;
		$date = \DateTime::createFromFormat('!'.self::$modeOpts[$this->mode]['format'], $src);
		/* if (!$date)
		{
			$date = \DateTime::createFromFormat('!'.self::dtfns, $src);
		}*/

		return $date;
	}
	/**
	* Loads HTTP data.
	* @return void
	*/
	public function loadHttpData(): void
	{
		$this->value->setDtFrom($this->parseDatetime($this->getHttpData(\Nette\Forms\Form::DATA_TEXT, self::NAME_SEPARATOR.'dtFrom')));
		$this->value->setDtTo($this->parseDatetime($this->getHttpData(\Nette\Forms\Form::DATA_TEXT, self::NAME_SEPARATOR.'dtTo')));
		$this->value->isNull = $this->getHttpData(\Nette\Forms\Form::DATA_TEXT, self::NAME_SEPARATOR.'isNull') != null;
	}

	static function register() 
	{
		FormContainer::extensionMethod('addDateTimeInterval', function($form, $name, $label)
		{
			$form[$name] = new DateTimeIntervalControl($label);
			return $form[$name];
		});
	}

}

/**
 * @property \DateTime|null dtFrom
 * @property \DateTime|null dtTo
 * @property \DateTime|null fixDtFrom
 * @property \DateTime|null fixDtTo
 * @property bool unlimited
 * @property bool closed
 */
class DateTimeInterval implements \JsonSerializable
{
	use \Nette\SmartObject;

	public $roundMode;

	protected $_dtFrom = null;

	protected $_dtTo = null;

	public $isNull = false;

	protected $mode = null;

	const modeInterval = 'interval';

	public function setDtFrom($dtFrom)
	{
		$this->_dtFrom = $dtFrom;
	}

	public function setDtTo($dtTo)
	{
		$this->_dtTo = $dtTo;
	}

	public function getDtFrom()
	{
		return $this->round($this->_dtFrom, false);
	}

	public function getDtTo()
	{
		return $this->round($this->_dtTo, true);
	}

	public function getFixDtFrom()
	{
		return $this->_dtFrom;
	}

	public function getFixDtTo()
	{
		return $this->_dtTo;
	}

	public function isUnlimited()
	{
		return !$this->dtFrom && !$this->dtTo;
	}

	public function isClosed()
	{
		return $this->dtFrom && $this->dtTo;
	}

	public function round(/*?\DateTime*/ $dt, bool $isEnd)
	{
		if ($dt == null) return null;
		$dt = clone $dt;
		switch($this->roundMode)
		{
			case 'm':
				list($h, $i) = explode(':', $dt->format('H:i'));
				$dt->setTime($h, $i, $isEnd ? 59 : 0);
				break;
			case 'd':
				if ($isEnd) $dt->setTime(23, 59, 59);
				else $dt->setTime(0, 0, 0);
				break;
		}
		return $dt;
	}

	public const serializeFormat = "Y-m-d\TH:i:s";

	public function jsonSerialize()
	{
		$res = [
			// 'roundMode' => $this->roundMode,
			'_dtFrom' => $this->_dtFrom ? $this->_dtFrom->format(self::serializeFormat) : null,
			'_dtTo' => $this->_dtTo ? $this->_dtTo->format(self::serializeFormat) : null,
			'mode' => $this->mode,
			'isNull' => $this->isNull,
		];
		return $res;
	}

	public function __construct($src = [])
	{
		if (is_string($src)) $src = json_decode($src, true);
		foreach(['roundMode', '_dtFrom', '_dtTo', 'mode', 'isNull'] as $k)
		{
			if (isset($src[$k]))
			{
				$v = $src[$k];
				if (in_array($k, ['_dtFrom', '_dtTo']))
				{
					if (is_array($v)) $v = new DateTime($v["date"]);
					else if (is_string($v)) $v = new DateTime($v);
				}
				$this->$k = $v;
			} 
		}
	}
}

