<?php

namespace Murdej\Forms;

use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;
use Nette\Utils\Html;
use Murdej\ProcI;
use Nette\Utils\Arrays;
use Nette\Utils\DateTime;
use DateTimeInterface;

/**
* Datetime range control.
*
* @author     Murdej
* @package    Nette\Extras
*/
class NumberIntervalControl extends \Nette\Forms\Controls\BaseControl
{
	use \Nette\SmartObject;

	public function getValue() : NumberInterval
	{
		return $this->value;
	}

	public $isDecimal = true;

	public $filterNull = false;

	public $decimalPlaces = null;

	public function setValue(/*DateTimeInterval */$value)
	{
		if ($value instanceof NumberInterval)
		{
			$this->value = $value;
		}
		else if (is_array($value))
		{
			$this->value = new NumberInterval($value);
			// list($this->value->dtFrom, $this->value->dtTo) = $value;
		}
	}

	public function __construct($label)
	{
		parent::__construct($label);
		$this->value = new NumberInterval();
	}

	protected function makeSubInput(string $tail, $value, array $attributes = [])
	{
		$subInput = clone parent::getControl();
		$subInput->addAttributes([
			"mu-id" => $tail,
			"value" => $value,
			'autocomplete' => "off",
			'name' => $this->getHtmlName().self::NAME_SEPARATOR.$tail,
			'id' => $this->getHtmlId().self::NAME_SEPARATOR.$tail,
		] + $attributes);

		return $subInput;
	}
	/**
	* Generates control's HTML element.
	* @return Html
	*/
	public function getControl() : Html
	{
		$el = Html::el('div')
			->addAttributes([
				"mu-widget" => "MurdejFormNumberInterval",
				'class' => 'MurdejFormNumberInterval'.($this->filterNull ? '-null' : '').' form-control',
			]);
		$nattrs = ['type' => 'number', 'step' => pow(10, -$this->decimalPlaces)];
		if ($this->filterNull) 
		{
			$el->addHtml(
				Html::el('label')
					->addHtml($this->makeSubInput(
						'isNull', 
						'1',
						['type' => 'checkbox', 'checked' => $this->value->isNull]
					))
					->addText('Nevyplněno')
					->setClass('is-null')
			);
		}
		$el->addHtml(
			Html::el('label')
				->addHtml(Html::el('span')->setText('Od'))
				->addHtml($this->makeSubInput(
					'from', 
					$this->value->from !== null ? $this->formatNumber($this->value->from) : '',
					$nattrs
				))
				->setAttribute("mu-id", "fromLabel")
			)
			->addHtml(
			Html::el('label')
				->addHtml(Html::el('span')->setText('do'))
				->addHtml($this->makeSubInput(
					'to', 
					$this->value->to !== null ? $this->formatNumber($this->value->to) : '',
					$nattrs
				))
				->setAttribute("mu-id", "toLabel")
			);

		return $el;
	}

	protected function parse($src) : ?float
	{
		if (!$src) return null;
		$num = (float)str_replace(",", ".", $src);

		return $num;
	}
	/**
	* Loads HTTP data.
	* @return void
	*/
	public function loadHttpData(): void
	{
		$this->value->from = $this->parse($this->getHttpData(\Nette\Forms\Form::DATA_TEXT, self::NAME_SEPARATOR.'from'));
		$this->value->to = $this->parse($this->getHttpData(\Nette\Forms\Form::DATA_TEXT, self::NAME_SEPARATOR.'to'));
		$this->value->isNull = $this->getHttpData(\Nette\Forms\Form::DATA_TEXT, self::NAME_SEPARATOR.'isNull') != null;
	}

	protected function formatNumber($num)
	{
		return $num;
	}

	static function register() 
	{
		FormContainer::extensionMethod('addNumberInterval', function($form, $name, $label)
		{
			return $form[$name] = new self($label);
		});
	}

}

/**
 * @property float|null from
 * @property float|null to
 * @property bool unlimited
 * @property bool closed
 */
class NumberInterval implements \JsonSerializable
{
	use \Nette\SmartObject;

	public $roundMode;

	protected $_from = null;

	protected $_to = null;
	
	public $isNull = false;

	public function setFrom($from)
	{
		$this->_from = $from;
	}

	public function setTo($to)
	{
		$this->_to = $to;
	}

	public function getFrom()
	{
		return $this->_from;
	}

	public function getTo()
	{
		return $this->_to;
	}

	public function isUnlimited()
	{
		return !$this->from && !$this->to;
	}

	public function isClosed()
	{
		return $this->from && $this->to;
	}

	public function jsonSerialize()
	{
		$res = [
			'_from' => $this->_from,
			'_to' => $this->_to,
			'isNull' => $this->isNull,
		];
		return $res;
	}

	public function __construct($src = [])
	{
		if (is_string($src)) $src = json_decode($src, true);
		foreach(['_from', '_to', 'isNull'] as $k)
		{
			if (isset($src[$k]))
			{
				$v = $src[$k];
				$this->$k = $v;
			} 
		}
	}
}

