<?php

namespace Murdej\Forms;

use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;
use Nette\Utils\Html;
use YaLinqo\Utils;

/*todo:
	+ Přejmenovat soubor
	+ Nové položky, (get|set)Value
	+ static::register add(Single|Multi)Selectize
	+ prompt
	+ composer
*/
/**
* Selectize input control.
*
* @author     Murdej
* @package    Nette\Extras
*
* @property mixed $value
* @property mixed $prompt
*/
class Selectize extends \Nette\Forms\Controls\BaseControl
{
	use \Nette\SmartObject;

	public $allowCreate = false;

	public $multiple;

	public $items = [];

	public $autocompleteUrl = null;

	public $newValuePrefix = '+';

	public $existingValuePrefix = '.';

	public $valueField = 'id';

	public $labelField = 'name';

	public $searchField = null; // 'name';

	public $templateListItem = '';

	public $templateSelectedItem = '';

	public $templateItem = '';

	protected $value;

	public $newValue;

	public $missingValuesCallback = null;

	protected $prompt = null;

	public function __construct($label, bool $multiple, array $items = [], bool $allowCreate = false, /*?string*/ $autocompleteUrl = null, $missingValuesCallback = null)
	{
		parent::__construct($label);
		$this->items = $items;
		$this->allowCreate = $allowCreate;
		$this->autocompleteUrl = $autocompleteUrl;
		$this->multiple = $multiple;
		$this->setMissingValuesCallback($missingValuesCallback);
	}

	/**
	* Generates control's HTML element.
	* @return Html
	*/
	public function getControl()
	{
		$input = clone parent::getControl();
		// $input->setName('select');
		$input->setName('select');
		// $input->type = 'text';
		// $input->name = $this->getHtmlName() . ($this->multiple ? '[]' : '');
		$input->multiple = $this->multiple;
		$opts = [
			'create' => $this->allowCreate,
			'newValuePrefix' => $this->newValuePrefix,
			'existingValuePrefix' => $this->existingValuePrefix,
			'autocompleteUrl' => $this->autocompleteUrl,
			'valueField' => $this->valueField,
			'labelField' => $this->labelField,
			'searchField' => $this->searchField
				?: (
					$this->templateListItem || $this->templateSelectedItem || $this->templateItem
					? 'text'
					: 'name'
				),
			/* 'templateListItem' => $this->templateListItem ? $this->makeJsTemplate($this->templateListItem) : null,
			'templateSelectedItem' => $this->templateSelectedItem ? $this->makeJsTemplate($this->templateSelectedItem) : null,
			'templateItem' => $this->templateItem ? $this->makeJsTemplate($this->templateItem) : null, */
		];
		$js = Html::el('script');
		$useJs = false;
		$sp = 'window.SelectizeTemplates';
		$js->addHtml("\nif(typeof $sp === 'undefined') $sp = {};\n;");
		foreach([
			'templateListItem' => $this->templateListItem ? $this->makeJsTemplate($this->templateListItem) : null,
			'templateSelectedItem' => $this->templateSelectedItem ? $this->makeJsTemplate($this->templateSelectedItem) : null,
			'templateItem' => $this->templateItem ? $this->makeJsTemplate($this->templateItem) : null, 
		] as $k => $code)
		{
			if ($code)
			{
				$useJs = true;
				$fn = 'f'.md5($code);
				$js->addHtml("$sp.$fn=function(item,escape){/*\nconsole.log($code);*/\nreturn $code;\n};\n");
				$opts[$k] = $fn;
			}
		}

		$missingItems = [];
		// Načíst chybějící položky
		if ($this->missingValuesCallback)
		{
			$missingItemValues = array_diff(
				$this->value
					? ($this->multiple ? $this->value : [$this->value])
					: [], // vybrané
				array_keys($this->items)
			);
			if ($missingItemValues) $missingItems = ($this->missingValuesCallback)($missingItemValues);
		}

		/* foreach(
			($this->prompt ? ['' => $this->prompt] : [])
			+ $this->items 
			+ $missingItems as $value => $title)
		{
			$option = Html::el('option');
			$option->value = $this->existingValuePrefix.$value;
			if ((is_array($this->value) && in_array($value, $this->value)) || ($value == $this->value)) $option->selected = true;
			$option->addText($title);

			$input->addHtml($option);
		}

		if ($this->newValue)
		{
			foreach($this->multiple ? $this->newValue : [$this->newValue] as $value)
			{
				$option = Html::el('option');
				$option->value = $this->newValuePrefix.$value;
				$option->selected = true;
				$option->addText($value);
	
				$input->addHtml($option);
			}
		} */
		$options = [];
		$selected = $this->multiple ? [] : null;
		$currValue = $this->value;
		if ($currValue instanceof \Nette\Utils\ArrayHash)
		{
			$tmp = [];
			foreach($currValue as $v) $tmp[] = $v;
			$currValue = $tmp;
		}

		foreach(
			$this->items 
			+ $missingItems as $value => $title)
		{
			$options[$this->existingValuePrefix.$value] = $title;
			if ((is_array($currValue) && in_array($value, $currValue)) || ($value == $currValue))
			{
				if ($this->multiple) $selected[] = $this->existingValuePrefix.$value;
				else $selected = $this->existingValuePrefix.$value;
			}
		}

		if ($this->newValue)
		{
			foreach($this->multiple ? $this->newValue : [$this->newValue] as $value)
			{
				$options[$this->newValuePrefix.$value] = $value;
				if ($this->multiple) $selected[] = $this->newValuePrefix.$value;
				else $selected = $this->newValuePrefix.$value;
			}
		}
		
		$opts['options'] = $options;
		$opts['items'] = $selected;
		$opts['placeholder'] = $this->prompt;
		$input->setAttribute('data-selectize', json_encode($opts));
		$input->setAttribute('class', 'selectize-loading');

		$el = Html::el();
		$el->addHtml($input);
		if ($useJs) $el->addHtml($js);

		return $el;
	}

	/**
	* Loads HTTP data.
	* @return void
	*/
	public function loadHttpData(): void
	{
		if ($this->multiple)
		{
			$this->newValue = [];
			$this->value = [];
			foreach($this->getHttpData(\Nette\Forms\Form::DATA_TEXT) as $v)
			{
				if ($v[0] == $this->newValuePrefix)
				{
					$this->newValue[] = substr($v, 1);
				}
				else
				{
					$this->value[] = substr($v, 1);
				}
			};
		}
		else
		{
			$v = $this->getHttpData(\Nette\Forms\Form::DATA_TEXT);
			if ($v)
			{
				if ($v[0] == $this->newValuePrefix)
				{
					$this->newValue = substr($v, 1);
				}
				else
				{
					$this->value = substr($v, 1);
				}
			}
		}
	}

	public function getHtmlName(): string
	{
		return parent::getHtmlName() . ($this->multiple ? '[]' : '');
	}

	public function getValue()
	{
		if ($this->multiple)
		{
			$res = $this->value;
			if ($this->newValue) $res['new'] = $this->newValue;

			return $res;
		}
		else
		{
			return $this->newValue ?: $this->value;
		}
	}

	public function setValue($value)
	{
		$this->value = $value;
		$this->newValue = null;
	}

	/**
	 * Set prompt item
	 */
	public function setPrompt($prompt)
	{
		$this->prompt = $prompt;
		return $this;
	}

	/**
	 * Returns prompt item
	 * @return mixed
	 */
	public function getPrompt()
	{
		return $this->prompt;
	}

	public function setMissingValuesCallback($callback)
	{
		$this->missingValuesCallback = $callback;
		return $this;
	}

	public function preload(array $values)
	{
		if ($this->missingValuesCallback)
		{
			$this->items += ($this->missingValuesCallback)($values);
		}
		else throw new \LogicException('missingValuesCallback must be set for preload()');
	}

	public function makeJsTemplate(string $template) : string
	{
		if (!\Nette\Utils\Strings::startsWith($template, "<div"))
		{
			$template = "<div>$template</div>";
		}
		$res = ''; // 'function(item, escape) { return ';
		//todo: validace {}
		foreach(preg_split('/[\{\}]/', $template) as $i => $str)
		{
			if ($i) $res .= '+';
			if ($i % 2)
			{
				$field = $str;
				$esc = $field[0] != '!';
				if (!$esc) $field = substr($field, 1);
				$f = 'item.'.$this->labelField.'.'.$field;
				$res .= '('.$f.' ? '
					.($esc ? 'escape(' : '')
					.$f
					.($esc ? ')' : '')
					.' : "")';
			}
			else $res .= '"'.addslashes($str).'"';
		}
		// $res .= ' + console.log('.$res.')';
		// $res .= ";}";

		return $res;
	}

	static function register() 
	{
		FormContainer::extensionMethod('addSelectizeMulti', function($form, $name, $label, array $items = [], bool $allowCreate = false, /*?string*/ $autocompleteUrl = null, $missingValuesCallback = null)
		{
			$form[$name] = new Selectize($label, true, $items, $allowCreate, $autocompleteUrl, $missingValuesCallback);
			return $form[$name];
		});
		FormContainer::extensionMethod('addSelectizeSingle', function($form, $name, $label, array $items = [], bool $allowCreate = false, /*?string*/ $autocompleteUrl = null, $missingValuesCallback = null)
		{
			$form[$name] = new Selectize($label, false, $items, $allowCreate, $autocompleteUrl, $missingValuesCallback);
			return $form[$name];
		});
		/* FormContainer::extensionMethod('addMultiSelectize', function($form, $name, $label, array $items = [], bool $allowCreate = false, /*?string* / $autocompleteUrl = null)
		{
			$this[$name] = new Selectize($label, $multiple, $items, $allowCreate, $autocompleteUrl);
		});*/
	}
}
