<?php

namespace Murdej;

use Nette\Application\UI\Form;
use Nette\Forms\IControl,
	Nette\Forms\Controls\BaseControl;
use Nette\Utils\Html;

class MuWidgetStub extends BaseControl 
{
	/** @var string */
	public $widgetName = null;

	/** @var string */
	public $widgetId = null;

	/** @var array */
	public $widgetParams = [];

	/** @var ?callable */
	public $serializeCallback = null;

	/** @var ?callable */
	public $deserializeCallback = null;
	
	/** @var ?callable */
	public $validateCallback = null;

	/** @var string */
	public $containerTag = "div";

	/** @var string */
	public $inputType = "hidden";

	public function __construct($label = null, string $widgetName, array $widgetParams = [], ?string $widgetId = null, $serializeCallback = null, $deserializeCallback = null, $validateCallback = null)
	{
		parent::__construct($label);

		$this->widgetName = $widgetName;
		$this->widgetParams = $widgetParams;
		$this->validateCallback = $validateCallback;
		$this->widgetId = $widgetId;
		$this->serializeCallback = $serializeCallback;
		$this->deserializeCallback = $deserializeCallback;
		// $this->setRequired("Musí být zadáno aspoň jedno pravidlo");
		if ($this->validateCallback) $this->addRule($this->validateCallback, "Opravte chyby v pravidlech.");
	}

	public function valueIsJson()
	{
		$this->serializeCallback = "json_encode";
		$this->deserializeCallback = function($v) { return json_decode($v, true); };
	}

	public function getControl() {
		$control = parent::getControl();
		$container = $this->containerTag ? Html::el($this->containerTag) : $control;
		if ($this->containerTag)
		{
			$control->addAttributes([
				"mu-id" => "input",
				"value" => $this->serializeCallback
					? ($this->serializeCallback)($this->value)
					: $this->value
			]);
			$container->addHtml($control);
		} 
		$control->setAttribute("type", $this->inputType);
		$container->addAttributes([
			"mu-widget" => $this->widgetName,
			"mu-id" => $this->widgetId,
			"mu-params" => $this->widgetParams ? json_encode($this->widgetParams) : null,
		]);
		$container = $this->prepareControl($container);

		return $container;
	}

	public function prepareControl($container)
	{
		return $container;
	}

	public function loadHttpData() : void
	{
		$rawValue = $this->getHttpData(Form::DATA_TEXT);
		$this->value = $this->deserializeCallback
			? ($this->deserializeCallback)($rawValue)
			: $rawValue;
	}
}